import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Modal } from 'antd'
import { DownOutlined, UpOutlined } from '@ant-design/icons'

import Utils from '@/utils/utils.js'
import { getChartViewForm } from './formconfig'

import ChartForm from './chartform'
import DragChartView from './dragchartview'
import './index.scss'

const { confirm } = Modal

class ChartGroupComponent extends Component {
  static propTpyes = {
    config: PropTypes.object,         // 菜单配置信息
    updatechartgroup: PropTypes.func  // 图表更新
  }

  state = {
    chartlist: null,      // 图表集
    card: null,           // 编辑中元素
    formlist: null,       // 表单信息
    modaltype: '',        // 模态框控制
    chartview: null       // 活动图表
  }

  /**
   * @description 图表初始化
   */
  UNSAFE_componentWillMount () {
    this.setState({
      chartlist: fromJS(this.props.config.charts).toJS(),
      chartview: this.props.config.charts[0].uuid
    })
  }

  /**
   * @description 图表信息修改时更新 chartlist
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { chartlist } = this.state

    if (nextProps.config && nextProps.config.charts && !is(fromJS(nextProps.config.charts), fromJS(chartlist))) {
      this.setState({
        chartlist: fromJS(nextProps.config.charts).toJS()
      })
    }
  }

  /**
   * @description 图表顺序调整，或拖拽添加
   */
  handleList = (list) => {
    const { config } = this.props

    this.setState({chartlist: list})
    this.props.updatechartgroup({...config, charts: list}, this.state.chartview)
  }

  /**
   * @description 取消保存，关闭模态框
   */
  editModalCancel = () => {
    this.setState({
      card: null,
      modaltype: ''
    })
  }

  /**
   * @description 添加或修改图表
   */
  handleChart = (item) => {
    const { config } = this.props

    let _type = 'editChart'
    if (!item) {
      _type = 'addChart'
      item = {
        uuid: Utils.getuuid(),
        label: '',
        title: '',
        chartType: 'line',
        icon: 'line-chart',
        Hide: 'false',
        blacklist: [],
        barSize: 35
      }
    }

    let _columns = [{
      uuid: 'x',
      value: '',
      text: '空',
      background: ''
    }]
    config.columns.forEach(col => {
      if (!['text', 'number'].includes(col.type)) return
      if (!col.marks) return

      let _marks = col.marks.filter(_mark => _mark.signType === 'card')

      if (_marks.length === 0) return

      _columns.push({
        uuid: col.uuid,
        value: col.field,
        text: col.label,
        background: _marks[0].color ? _marks[0].color[1] : ''
      })
    })

    if (item.chartType === 'card') { // 检验已添加背景是否修改
      if (item.bgfield) {
        let _column = _columns.filter(col => col.value === item.bgfield)[0]
        if (_column) {
          item.background = _column.background
        } else {
          item.bgfield = ''
          item.background = ''
        }
      } else {
        item.background = ''
      }
    }

    let actions = config.action.filter(item => item.OpenType === 'excelOut' || (item.OpenType === 'excelIn' && item.Ot === 'notRequired'))
    actions = actions.map(cell => ({value: cell.uuid, text: cell.label}))

    if (item.actions && item.actions.length > 0) {
      let keys = actions.map(cell => cell.value)
      item.actions = item.actions.filter(cell => keys.includes(cell))
    }

    let extraActions = config.action.filter(item => ['pop', 'prompt', 'exec'].includes(item.OpenType) && item.Ot === 'notRequired')
    extraActions = extraActions.map(cell => ({value: cell.uuid, text: cell.label}))

    if (item.extraAction && extraActions.filter(cell => cell.value === item.extraAction).length === 0) {
      item.extraAction = ''
    }

    this.setState({
      card: item,
      modaltype: _type,
      formlist: getChartViewForm(item, _columns, actions, extraActions)
    })
  }

  /**
   * @description 图表信息修改后，提交保存
   */
  submitChart = () => {
    const { config } = this.props
    const { modaltype } = this.state
    let _chartlist = fromJS(this.state.chartlist).toJS()
    let _chartview = this.state.chartview

    this.chartFormRef.handleConfirm().then(res => {
      if (modaltype === 'addChart') {
        _chartlist.push(res)
        _chartview = res.uuid
      } else {
        _chartlist = _chartlist.map(item => {
          if (item.uuid === res.uuid) {
            if (!is(fromJS(item), fromJS(res))) {
              let _element = document.getElementById(res.uuid)
              if (_element) {
                _element.innerHTML = ''
              }
            }
            return res
          }
          return item
        })
      }

      this.setState({
        card: null,
        chartview: _chartview,
        modaltype: '',
        chartlist: _chartlist
      })

      this.props.updatechartgroup({...config, charts: _chartlist}, _chartview)
    })
  }

  /**
   * @description 图表展开收起切换
   */
  onChartChange = (e) => {
    const { config } = this.props
    e.stopPropagation()

    this.props.updatechartgroup({...config, expand: !config.expand}, this.state.chartview)
  }

  /**
   * @description 图表删除
   */
  deletechart = (plot) => {
    const { config } = this.props
    let _this = this

    confirm({
      content: `确定删除 ${plot.title} ？`,
      onOk() {
        let _chartlist = fromJS(_this.state.chartlist).toJS()
        let _chartview = _this.state.chartview

        _chartlist = _chartlist.filter(item => item.uuid !== plot.uuid)

        if (_chartview === plot.uuid) {
          _chartview = _chartlist[0].uuid
        }

        _this.setState({
          chartlist: _chartlist,
          chartview: _chartview
        })
        _this.props.updatechartgroup({...config, charts: _chartlist}, _chartview)
      },
      onCancel() {}
    })
  }

  changetabview = (id) => {
    const { config } = this.props

    this.setState({
      chartview: id
    })

    this.props.updatechartgroup(config, id)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  // 已弃用，不在开放添加通道
  render() {
    const { config } = this.props
    const { chartlist, modaltype, card, chartview } = this.state

    return (
      <div className="model-table-chartview-list">
        {/* <PlusOutlined onClick={() => this.handleChart()} /> */}
        {chartlist.length > 1 ? (config.expand ? <UpOutlined onClick={this.onChartChange} /> : <DownOutlined onClick={this.onChartChange} />) : null}
        {chartlist.length > 1 ? <DragChartView
          activeKey={chartview}
          list={chartlist}
          expand={config.expand}
          handleList={this.handleList}
          changetabview={this.changetabview}
          handleMenu={this.handleChart}
          deleteMenu={this.deletechart}
        /> : null}
        {/* 图表信息编辑 */}
        <Modal
          title={modaltype === 'addChart' ? '图表-添加' : '图标-编辑'}
          visible={modaltype === 'addChart' || modaltype === 'editChart'}
          width={800}
          maskClosable={false}
          onOk={this.submitChart}
          onCancel={this.editModalCancel}
          destroyOnClose
        >
          <ChartForm
            card={card}
            formlist={this.state.formlist}
            inputSubmit={this.submitChart}
            wrappedComponentRef={(inst) => this.chartFormRef = inst}
          />
        </Modal>
      </div>
    )
  }
}

export default ChartGroupComponent